/*=========================================================================

  Program:   Ionization FRont Interactive Tool (IFRIT)
  Language:  C++


Copyright (c) 2002-2006 Nick Gnedin 
All rights reserved.

This file may be distributed and/or modified under the terms of the
GNU General Public License version 2 as published by the Free Software
Foundation and appearing in the file LICENSE.GPL included in the
packaging of this file.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

=========================================================================*/


#include "iqtdialogsubject.h"


#include "ierror.h"

#include "iggdialog.h"
#include "iggframe.h"
#include "iggmainwindow.h"

#include "iqtassistant.h"
#include "iqtframesubject.h"
#include "iqtmainwindowsubject.h"
#include "iqtwindowhelper.h"

#ifdef IQT_3
#include <qdialog.h>
#include <qlayout.h>
#include <qpushbutton.h>
#include <qtooltip.h>
#else
#include <QtGui/QCloseEvent>
#include <QtGui/QDialog>
#include <QtGui/QGridLayout>
#include <QtGui/QPushButton>
#include <QtGui/QToolTip>
#endif

#include "iggparameter.h"
using namespace iggParameter;


//
//  Main class
//
iqtDialogSubject::iqtDialogSubject(iggDialog *owner, const ibgWindowSubject *base, unsigned int mode, const iImage *icon, const iString &title) : 
#ifdef IQT_3
	QDialog((mode&_DialogUnattached)?0:iPointerCast<iqtWindowHelper,ibgWindowHelper>(base->GetHelper())->GetWidget(),"",false,iqtAssistant::GetFlags(iqtAssistant::_Dialog,mode)), 
#else
	QDialog((mode&_DialogUnattached)?0:iPointerCast<iqtWindowHelper,ibgWindowHelper>(base->GetHelper())->GetWidget(),iqtAssistant::GetFlags(iqtAssistant::_Dialog,mode)), 
#endif
	ibgDialogSubject(owner,base,mode,icon,title)
{
	mWindowHelper = new iqtWindowHelper(this,icon,title); IERROR_ASSERT(mWindowHelper);

	mMode = mode;
	QGridLayout *layout = iqtAssistant::NewLayout(this,1);

	if((mode & _DialogBlocking) == 0) this->setSizeGripEnabled(true); else
	{
#ifndef IQT_3
		 this->setAttribute(Qt::WA_ShowModal); 
#endif
	}
	if((mode & _DialogNoTitleBar) != 0)
	{
		this->setFocusPolicy(QFocus::NoFocus);
		iqtAssistant::SetBackground(this,0);
	}
}


iqtDialogSubject::~iqtDialogSubject()
{
}


void iqtDialogSubject::Show(bool s)
{
	if(s)
	{
		if(mMode & _DialogWithEventLoop)
		{
			this->exec();
		}
		else
		{
			this->show();
			if(!this->isHidden()) this->raise();
		}
	}
	else this->hide();
}


void iqtDialogSubject::SetFrame(ibgFrameSubject *frame0)
{
	iqtFrameSubject *frame = iPointerCast<iqtFrameSubject,ibgFrameSubject>(frame0);

	if(frame->parentWidget() != this)
	{
		iqtAssistant::SetParent(frame,this);
		iPointerCast<QGridLayout,QLayout>(this->layout())->addWidget(frame,0,0);
	}
	if((mMode & _DialogNoTitleBar) != 0)
	{
#ifdef IQT_3
		frame->setFrameShape(QFrame::Box);
		frame->setFrameShadow(QFrame::Raised);
		frame->setLineWidth(3);
		frame->setMidLineWidth(2);
#else
		frame->setFlat(false);
#endif
		frame->layout()->setMargin(5);
	}
}


void iqtDialogSubject::closeEvent(QCloseEvent *e)
{
	this->OnClose();
	e->ignore();
}


void iqtDialogSubject::keyPressEvent(QKeyEvent *e)
{
	//
	//  Block closing of dialogs on Enter
	//
	if(e->key() == Qt::Key_Return) e->accept(); else QDialog::keyPressEvent(e);
}
